@@ -40,6 +40,7 @@ enum
     COL_CATEGORY,
     COL_PRIORITY,
     COL_PRIORITY_TEXT,
+    COL_COLORS,
     NUM_COLUMNS
 };
 
@@ -51,8 +52,4 @@ struct menu_map
 
 extern struct menu_map state_map[];
 
-#ifdef IS_HILDON
-GtkWidget *main_appview;  
-#endif
-
 #endif
